"""Python wrapper around the Zoom.us REST API"""

from __future__ import absolute_import, unicode_literals

from zoomus.client import ZoomClient
from zoomus.util import API_VERSION_1, API_VERSION_2


__all__ = ["API_VERSION_1", "API_VERSION_2", "ZoomClient"]
__version__ = "1.1.3"



"""Zoom.us REST API Python Client -- Report component"""

from __future__ import absolute_import

from zoomus import util
from zoomus.components import base


class ReportComponent(base.BaseComponent):
    """Component dealing with all report related matters"""

    def get_account_report(self, **kwargs):
        util.require_keys(kwargs, ["start_time", "end_time"], kwargs)
        kwargs["from"] = util.date_to_str(kwargs["start_time"])
        del kwargs["start_time"]
        kwargs["to"] = util.date_to_str(kwargs["end_time"])
        del kwargs["end_time"]
        return self.post_request("/report/getaccountreport", params=kwargs)

    def get_user_report(self, **kwargs):
        util.require_keys(kwargs, ["start_time", "end_time"], kwargs)
        kwargs["from"] = util.date_to_str(kwargs["start_time"])
        del kwargs["start_time"]
        kwargs["to"] = util.date_to_str(kwargs["end_time"])
        del kwargs["end_time"]
        return self.post_request("/report/getuserreport", params=kwargs)

    def get_daily_report(self, **kwargs):
        util.require_keys(kwargs, ["month", "year"], kwargs)
        return self.post_request("/report/getdailyreport", params=kwargs)


class ReportComponentV2(base.BaseComponent):
    def get_user_report(self, **kwargs):
        util.require_keys(kwargs, ["user_id", "start_time", "end_time"])
        kwargs["from"] = util.date_to_str(kwargs["start_time"])
        del kwargs["start_time"]
        kwargs["to"] = util.date_to_str(kwargs["end_time"])
        del kwargs["end_time"]
        return self.get_request(
            "/report/users/{}/meetings".format(kwargs.get("user_id")), params=kwargs
        )

    def get_account_report(self, **kwargs):
        util.require_keys(kwargs, ["start_time", "end_time"])
        kwargs["from"] = util.date_to_str(kwargs["start_time"])
        del kwargs["start_time"]
        kwargs["to"] = util.date_to_str(kwargs["end_time"])
        del kwargs["end_time"]
        return self.get_request("/report/users", params=kwargs)

    def get_daily_report(self, **kwargs):
        util.require_keys(kwargs, ["month", "year"])
        return self.get_request("/report/daily", params=kwargs)



"""Zoom.us REST API Python Client"""

from __future__ import absolute_import, unicode_literals

from zoomus import components, util
from zoomus.util import API_VERSION_1, API_VERSION_2


API_BASE_URIS = {
    API_VERSION_1: "https://api.zoom.us/v1",
    API_VERSION_2: "https://api.zoom.us/v2",
}

COMPONENT_CLASSES = {
    API_VERSION_1: {
        "user": components.user.UserComponent,
        "meeting": components.meeting.MeetingComponent,
        "report": components.report.ReportComponent,
        "webinar": components.webinar.WebinarComponent,
        "recording": components.recording.RecordingComponent,
    },
    API_VERSION_2: {
        "user": components.user.UserComponentV2,
        "meeting": components.meeting.MeetingComponentV2,
        "past_meeting": components.past_meeting.PastMeetingComponentV2,
        "report": components.report.ReportComponentV2,
        "webinar": components.webinar.WebinarComponentV2,
        "recording": components.recording.RecordingComponentV2,
    },
}


class ZoomClient(util.ApiClient):
    """Zoom.us REST API Python Client"""

    """Base URL for Zoom API"""

    def __init__(
        self, api_key, api_secret, data_type="json", timeout=15, version=API_VERSION_2
    ):
        """Create a new Zoom client

        :param api_key: The Zooom.us API key
        :param api_secret: The Zoom.us API secret
        :param data_type: The expected return data type. Either 'json' or 'xml'
        :param timeout: The time out to use for API requests
        """
        try:
            BASE_URI = API_BASE_URIS[version]
            self.components = COMPONENT_CLASSES[version].copy()
        except KeyError:
            raise RuntimeError("API version not supported: %s" % version)

        super(ZoomClient, self).__init__(base_uri=BASE_URI, timeout=timeout)

        # Setup the config details
        self.config = {
            "api_key": api_key,
            "api_secret": api_secret,
            "data_type": data_type,
            "version": version,
            "token": util.generate_jwt(api_key, api_secret),
        }

        # Instantiate the components
        for key in self.components.keys():
            self.components[key] = self.components[key](
                base_uri=BASE_URI, config=self.config
            )

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        return

    def refresh_token(self):
        self.config["token"] = (
            util.generate_jwt(self.config["api_key"], self.config["api_secret"]),
        )

    @property
    def api_key(self):
        """The Zoom.us api_key"""
        return self.config.get("api_key")

    @api_key.setter
    def api_key(self, value):
        """Set the api_key"""
        self.config["api_key"] = value
        self.refresh_token()

    @property
    def api_secret(self):
        """The Zoom.us api_secret"""
        return self.config.get("api_secret")

    @api_secret.setter
    def api_secret(self, value):
        """Set the api_secret"""
        self.config["api_secret"] = value
        self.refresh_token()

    @property
    def meeting(self):
        """Get the meeting component"""
        return self.components.get("meeting")

    @property
    def report(self):
        """Get the report component"""
        return self.components.get("report")

    @property
    def user(self):
        """Get the user component"""
        return self.components.get("user")

    @property
    def webinar(self):
        """Get the webinar component"""
        return self.components.get("webinar")

    @property
    def recording(self):
        """Get the recording component"""
        return self.components.get("recording")



"""Zoom.us REST API Python Client"""

from __future__ import absolute_import

from zoomus import util
from zoomus.components import base


class PastMeetingComponentV2(base.BaseComponent):
    def list(self, **kwargs):
        util.require_keys(kwargs, "meeting_id")
        kwargs["meeting_id"] = util.encode_uuid(kwargs.get("meeting_id"))
        return self.get_request(
            "/past_meetings/{}/instances".format(kwargs.get("meeting_id")),
            params=kwargs,
        )

    def get(self, **kwargs):
        util.require_keys(kwargs, "meeting_id")
        kwargs["meeting_id"] = util.encode_uuid(kwargs.get("meeting_id"))
        return self.get_request(
            "/past_meetings/{}".format(kwargs.get("meeting_id")), params=kwargs
        )

    def get_participants(self, **kwargs):
        util.require_keys(kwargs, "meeting_id")
        kwargs["meeting_id"] = util.encode_uuid(kwargs.get("meeting_id"))
        return self.get_request(
            "/past_meetings/{}/participants".format(kwargs.get("meeting_id")),
            params=kwargs,
        )



"""Zoom.us REST API Python Client -- Recording component"""
from zoomus import util
from zoomus.components import base


class RecordingComponent(base.BaseComponent):
    """Component dealing with all recording related matters"""

    def list(self, **kwargs):
        util.require_keys(kwargs, "host_id")
        start = kwargs.pop("start", None)
        if start:
            kwargs["from"] = util.date_to_str(start)
        end = kwargs.pop("end", None)
        if end:
            kwargs["to"] = util.date_to_str(end)
        return self.post_request("/recording/list", params=kwargs)

    def delete(self, **kwargs):
        util.require_keys(kwargs, ["meeting_id"])
        return self.post_request("/recording/delete", params=kwargs)

    def get(self, **kwargs):
        util.require_keys(kwargs, ["meeting_id"])
        return self.post_request("/recording/get", params=kwargs)


class RecordingComponentV2(base.BaseComponent):
    """Component dealing with all recording related matters"""

    def list(self, **kwargs):
        util.require_keys(kwargs, "user_id")
        start = kwargs.pop("start", None)
        if start:
            kwargs["from"] = util.date_to_str(start)
        end = kwargs.pop("end", None)
        if end:
            kwargs["to"] = util.date_to_str(end)
        return self.get_request(
            "/users/{}/recordings".format(kwargs.get("user_id")), params=kwargs
        )

    def get(self, **kwargs):
        util.require_keys(kwargs, "meeting_id")
        return self.get_request(
            "/meetings/{}/recordings".format(kwargs.get("meeting_id")), params=kwargs
        )

    def delete(self, **kwargs):
        util.require_keys(kwargs, "meeting_id")
        return self.delete_request(
            "/meetings/{}/recordings".format(kwargs.get("meeting_id")), params=kwargs
        )



from __future__ import print_function, unicode_literals

from setuptools import setup
import codecs
import os
import re

here = os.path.abspath(os.path.dirname(__file__))


def read(file_paths, default=""):
    # intentionally *not* adding an encoding option to open
    try:
        with codecs.open(os.path.join(here, *file_paths), "r") as fh:
            return fh.read()
    except Exception:
        return default


def find_version(file_paths):
    version_file = read(file_paths)
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]", version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


description = "Python client library for Zoom.us REST API v1 and v2"
long_description = read("README.md", default=description)

setup(
    name="zoomus",
    version=find_version(["zoomus", "__init__.py"]),
    url="https://github.com/prschmid/zoomus",
    license="Apache Software License",
    author="Zoomus Contributors",
    install_requires=["requests", "PyJWT"],
    author_email="zoomus@googlegroups.com",
    description=description,
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=["zoomus", "zoomus.components"],
    include_package_data=True,
    platforms="any",
    zip_safe=False,
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Development Status :: 5 - Production/Stable",
        "Natural Language :: English",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Topic :: Internet",
        "Topic :: Office/Business",
        "Topic :: Software Development :: Libraries",
    ],
)



"""Zoom.us REST API Python Client -- User component"""

from __future__ import absolute_import

from zoomus import util
from zoomus.components import base


class UserComponent(base.BaseComponent):
    """Component dealing with all user related matters"""

    def list(self, **kwargs):
        return self.post_request("/user/list", params=kwargs)

    def pending(self, **kwargs):
        return self.post_request("/user/pending", params=kwargs)

    def create(self, **kwargs):
        return self.post_request("/user/create", params=kwargs)

    def update(self, **kwargs):
        util.require_keys(kwargs, "id")
        return self.post_request("/user/update", params=kwargs)

    def delete(self, **kwargs):
        util.require_keys(kwargs, "id")
        return self.post_request("/user/delete", params=kwargs)

    def cust_create(self, **kwargs):
        util.require_keys(kwargs, ["type", "email"])
        return self.post_request("/user/custcreate", params=kwargs)

    def get(self, **kwargs):
        util.require_keys(kwargs, "id")
        return self.post_request("/user/get", params=kwargs)

    def get_by_email(self, **kwargs):
        util.require_keys(kwargs, ["email", "login_type"])
        return self.post_request("/user/getbyemail", params=kwargs)


class UserComponentV2(base.BaseComponent):
    def list(self, **kwargs):
        return self.get_request("/users", params=kwargs)

    def create(self, **kwargs):
        return self.post_request("/users", data=kwargs)

    def update(self, **kwargs):
        util.require_keys(kwargs, "id")
        return self.patch_request("/users/{}".format(kwargs.get("id")), data=kwargs)

    def delete(self, **kwargs):
        util.require_keys(kwargs, "id")
        return self.delete_request("/users/{}".format(kwargs.get("id")), params=kwargs)

    def get(self, **kwargs):
        util.require_keys(kwargs, "id")
        return self.get_request("/users/{}".format(kwargs.get("id")), params=kwargs)



"""Zoom.us REST API Python Client"""

from __future__ import absolute_import

from zoomus import util
from zoomus.components import base


class MeetingComponent(base.BaseComponent):
    """Component dealing with all meeting related matters"""

    def list(self, **kwargs):
        util.require_keys(kwargs, "host_id")
        if kwargs.get("start_time"):
            kwargs["start_time"] = util.date_to_str(kwargs["start_time"])
        return self.post_request("/meeting/list", params=kwargs)

    def create(self, **kwargs):
        util.require_keys(kwargs, ["host_id", "topic", "type"])
        if kwargs.get("start_time"):
            kwargs["start_time"] = util.date_to_str(kwargs["start_time"])
        return self.post_request("/meeting/create", params=kwargs)

    def update(self, **kwargs):
        util.require_keys(kwargs, ["id", "host_id"])
        if kwargs.get("start_time"):
            kwargs["start_time"] = util.date_to_str(kwargs["start_time"])
        return self.post_request("/meeting/update", params=kwargs)

    def delete(self, **kwargs):
        util.require_keys(kwargs, ["id", "host_id"])
        return self.post_request("/meeting/delete", params=kwargs)

    def end(self, **kwargs):
        util.require_keys(kwargs, ["id", "host_id"])
        return self.post_request("/meeting/end", params=kwargs)

    def get(self, **kwargs):
        util.require_keys(kwargs, ["id", "host_id"])
        return self.post_request("/meeting/get", params=kwargs)


class MeetingComponentV2(base.BaseComponent):
    def list(self, **kwargs):
        util.require_keys(kwargs, "user_id")
        return self.get_request(
            "/users/{}/meetings".format(kwargs.get("user_id")), params=kwargs
        )

    def create(self, **kwargs):
        util.require_keys(kwargs, "user_id")
        if kwargs.get("start_time"):
            kwargs["start_time"] = util.date_to_str(kwargs["start_time"])
        return self.post_request(
            "/users/{}/meetings".format(kwargs.get("user_id")), data=kwargs
        )

    def get(self, **kwargs):
        util.require_keys(kwargs, "id")
        return self.get_request("/meetings/{}".format(kwargs.get("id")), params=kwargs)

    def update(self, **kwargs):
        util.require_keys(kwargs, "id")
        if kwargs.get("start_time"):
            kwargs["start_time"] = util.date_to_str(kwargs["start_time"])
        return self.patch_request("/meetings/{}".format(kwargs.get("id")), data=kwargs)

    def delete(self, **kwargs):
        util.require_keys(kwargs, "id")
        return self.delete_request(
            "/meetings/{}".format(kwargs.get("id")), params=kwargs
        )
